@@ -11,9 +11,10 @@ from asn1crypto import (
     cms, tsp, ocsp as asn1_ocsp, pdf as asn1_pdf, crl as asn1_crl, x509,
 )
 from asn1crypto.x509 import Certificate
+from cryptography.hazmat.primitives import hashes
 
-from certvalidator import ValidationContext, CertificateValidator
-from certvalidator.path import ValidationPath
+from pyhanko_certvalidator import ValidationContext, CertificateValidator
+from pyhanko_certvalidator.path import ValidationPath
 
 from pyhanko.pdf_utils import generic, misc
 from pyhanko.pdf_utils.generic import pdf_name
@@ -35,7 +36,8 @@ from .general import (
     SignatureStatus, find_cms_attribute,
     UnacceptableSignerError, KeyUsageConstraints,
     SignatureValidationError,
-    validate_sig_integrity,
+    validate_sig_integrity, DEFAULT_WEAK_HASH_ALGORITHMS,
+    get_pyca_cryptography_hash,
 )
 from .timestamps import TimestampSignatureStatus
 
@@ -46,7 +48,8 @@ __all__ = [
     'apply_adobe_revocation_info', 'get_timestamp_chain',
     'read_certification_data', 'validate_pdf_ltv_signature',
     'validate_pdf_signature', 'validate_cms_signature',
-    'collect_validation_info', 'add_validation_info',
+    'validate_pdf_timestamp', 'collect_validation_info',
+    'add_validation_info',
     'ValidationInfoReadingError', 'SigSeedValueValidationError'
 ]
 
@@ -58,6 +61,11 @@ class ValidationInfoReadingError(ValueError):
     pass
 
 
+class NoDSSFoundError(ValidationInfoReadingError):
+    def __init__(self):
+        super().__init__("No DSS found")
+
+
 class SigSeedValueValidationError(SignatureValidationError):
     """Error validating a signature's seed value constraints."""
 
@@ -126,11 +134,16 @@ def _validate_cms_signature(signed_data: cms.SignedData,
     """
     signer_info, cert, other_certs = _extract_signer_info_and_certs(signed_data)
 
+    weak_hash_algos = None
+    if validation_context is not None:
+        weak_hash_algos = validation_context.weak_hash_algos
+    if weak_hash_algos is None:
+        weak_hash_algos = DEFAULT_WEAK_HASH_ALGORITHMS
+
     signature_algorithm: cms.SignedDigestAlgorithm = \
         signer_info['signature_algorithm']
     mechanism = signature_algorithm['algorithm'].native
     md_algorithm = signer_info['digest_algorithm']['algorithm'].native
-
     expected_content_type = 'data'
     if raw_digest is None:
         # this means that there should be encapsulated data
@@ -138,12 +151,15 @@ def _validate_cms_signature(signed_data: cms.SignedData,
         expected_content_type = eci['content_type'].native
 
         raw = eci['content'].parsed.dump()
-        raw_digest = getattr(hashlib, md_algorithm)(raw).digest()
+        md_spec = get_pyca_cryptography_hash(md_algorithm)
+        md = hashes.Hash(md_spec)
+        md.update(raw)
+        raw_digest = md.finalize()
 
     # first, do the cryptographic identity checks
     intact, valid = validate_sig_integrity(
         signer_info, cert, expected_content_type=expected_content_type,
-        actual_digest=raw_digest,
+        actual_digest=raw_digest, weak_hash_algorithms=weak_hash_algos
     )
 
     # if the data being encapsulated by the signature is itself invalid,
@@ -353,7 +369,8 @@ class PdfSignatureStatus(ModificationInfo, SignatureStatus):
             timestamp_ok = ts.valid and ts.trusted
         return (
             self.valid and self.trusted and self.seed_value_ok
-            and self.docmdp_ok and timestamp_ok
+            and (self.docmdp_ok or self.modification_level is None)
+            and timestamp_ok
         )
 
     @property
@@ -420,7 +437,6 @@ class PdfSignatureStatus(ModificationInfo, SignatureStatus):
         if self.coverage == SignatureCoverageLevel.ENTIRE_FILE:
             modification_str = "The signature covers the entire file."
         else:
-            modlvl_string = "Some modifications may be illegitimate"
             if self.modification_level is not None:
                 if self.modification_level == ModificationLevel.LTA_UPDATES:
                     modlvl_string = \
@@ -430,12 +446,16 @@ class PdfSignatureStatus(ModificationInfo, SignatureStatus):
                         "All modifications relate to signing and form filling "
                         "operations"
                     )
-            modification_str = (
-                "The signature does not cover the entire file.\n"
-                f"{modlvl_string}, and they appear to be "
-                f"{'' if self.docmdp_ok else 'in'}compatible with the "
-                "current document modification policy."
-            )
+                else:
+                    modlvl_string = "Some modifications may be illegitimate"
+                modification_str = (
+                    "The signature does not cover the entire file.\n"
+                    f"{modlvl_string}, and they appear to be "
+                    f"{'' if self.docmdp_ok else 'in'}compatible with the "
+                    "current document modification policy."
+                )
+            else:
+                modification_str = "Incremental update analysis was skipped"
 
         validity_info = (
             "The signature is cryptographically "
@@ -642,6 +662,18 @@ class EmbeddedPdfSignature:
         self._integrity_checked = False
         self.fq_name = fq_name
 
+    @property
+    def sig_object_type(self) -> generic.NameObject:
+        """
+        Returns the type of the embedded signature object.
+        For ordinary signatures, this will be ``/Sig``.
+        In the case of a document timestamp, ``/DocTimeStamp`` is returned.
+
+        :return:
+            A PDF name object describing the type of signature.
+        """
+        return self.sig_object.get('/Type', pdf_name('/Sig'))
+
     @property
     def field_name(self):
         """
@@ -765,7 +797,8 @@ class EmbeddedPdfSignature:
     def docmdp_level(self) -> Optional[MDPPerm]:
         """
         :return:
-            The document modification policy required by this signature.
+            The document modification policy required by this signature or
+            its Lock dictionary.
 
             .. warning::
                 This does not take into account the DocMDP requirements of
@@ -778,11 +811,16 @@ class EmbeddedPdfSignature:
                 the earlier signature with the stricter DocMDP policy.
 
         """
-        # TODO fall back to reading /Lock in case the signing software
-        #  ignored the /Lock dictionary when building up the signature object
         if self._docmdp_queried:
             return self._docmdp
         docmdp = _extract_docmdp_for_sig(signature_obj=self.sig_object)
+
+        if docmdp is None:
+            try:
+                lock_dict = self.sig_field['/Lock']
+                docmdp = MDPPerm(lock_dict['/P'])
+            except KeyError:
+                pass
         self._docmdp = docmdp
         self._docmdp_queried = True
         return docmdp
@@ -821,7 +859,8 @@ class EmbeddedPdfSignature:
         if self.external_digest is not None:
             return self.external_digest
 
-        md = getattr(hashlib, self.external_md_algorithm)()
+        md_spec = get_pyca_cryptography_hash(self.external_md_algorithm)
+        md = hashes.Hash(md_spec)
         stream = self.reader.stream
 
         # compute the digest
@@ -836,7 +875,7 @@ class EmbeddedPdfSignature:
             total_len += chunk_len
 
         self.total_len = total_len
-        self.external_digest = digest = md.digest()
+        self.external_digest = digest = md.finalize()
         return digest
 
     def compute_tst_digest(self) -> Optional[bytes]:
@@ -868,8 +907,10 @@ class EmbeddedPdfSignature:
         tst_md_algorithm = mi['hash_algorithm']['algorithm'].native
 
         signature_bytes = self.signer_info['signature'].native
-        md = getattr(hashlib, tst_md_algorithm)(signature_bytes)
-        self.tst_signature_digest = digest = md.digest()
+        tst_md_spec = get_pyca_cryptography_hash(tst_md_algorithm)
+        md = hashes.Hash(tst_md_spec)
+        md.update(signature_bytes)
+        self.tst_signature_digest = digest = md.finalize()
         return digest
 
     def evaluate_signature_coverage(self) -> SignatureCoverageLevel:
@@ -1180,11 +1221,8 @@ def validate_pdf_signature(embedded_sig: EmbeddedPdfSignature,
     """
 
     sig_object = embedded_sig.sig_object
-    try:
-        if sig_object['/Type'] != '/Sig':
-            raise SignatureValidationError("Signature object type must be /Sig")
-    except KeyError:
-        pass
+    if embedded_sig.sig_object_type != '/Sig':
+        raise SignatureValidationError("Signature object type must be /Sig")
 
     # check whether the subfilter type is one we support
     subfilter_str = sig_object.get('/SubFilter', None)
@@ -1257,21 +1295,13 @@ def validate_pdf_timestamp(embedded_sig: EmbeddedPdfSignature,
         The status of the PDF timestamp in question.
     """
 
-    sig_object = embedded_sig.sig_object
-    invalid_obj_type = False
-    try:
-        if sig_object['/Type'] != '/DocTimeStamp':
-            invalid_obj_type = True
-    except KeyError:
-        invalid_obj_type = True
-
-    if invalid_obj_type:
+    if embedded_sig.sig_object_type != '/DocTimeStamp':
         raise SignatureValidationError(
             "Signature object type must be /DocTimeStamp"
         )
 
     # check whether the subfilter type is one we support
-    subfilter_str = sig_object.get('/SubFilter', None)
+    subfilter_str = embedded_sig.sig_object.get('/SubFilter', None)
     _validate_subfilter(
         subfilter_str, (SigSeedSubFilter.ETSI_RFC3161,),
         "%s is not a recognized SubFilter type for timestamps."
@@ -1419,9 +1449,12 @@ def _establish_timestamp_trust_lta(reader, bootstrap_validation_context,
         )
         # read the DSS at the current revision into a new
         # validation context object
-        current_vc = DocumentSecurityStore.read_dss(
-            reader.get_historical_resolver(emb_timestamp.signed_revision)
-        ).as_validation_context(validation_context_kwargs)
+        try:
+            current_vc = DocumentSecurityStore.read_dss(
+                reader.get_historical_resolver(emb_timestamp.signed_revision)
+            ).as_validation_context(validation_context_kwargs)
+        except NoDSSFoundError:
+            current_vc = ValidationContext(**validation_context_kwargs)
 
     return emb_timestamp, ts_status, ts_count + 1, current_vc
 
@@ -1449,7 +1482,7 @@ def validate_pdf_ltv_signature(embedded_sig: EmbeddedPdfSignature,
     :param validation_type:
         Validation profile to use.
     :param validation_context_kwargs:
-        Keyword args to instantiate :class:`.certvalidator.ValidationContext`
+        Keyword args to instantiate :class:`.pyhanko_certvalidator.ValidationContext`
         objects needed over the course of the validation.
     :param bootstrap_validation_context:
         Validation context used to validate the current timestamp.
@@ -2051,17 +2084,19 @@ class DocumentSecurityStore:
         certs = list(self._load_certs()) + extra_certs
 
         if include_revinfo:
-            ocsps = validation_context_kwargs['ocsps'] = []
+            ocsps = list(validation_context_kwargs.pop('ocsps', ()))
             for ocsp_ref in self.ocsps:
                 ocsp_stream: generic.StreamObject = ocsp_ref.get_object()
                 resp = asn1_ocsp.OCSPResponse.load(ocsp_stream.data)
                 ocsps.append(resp)
+            validation_context_kwargs['ocsps'] = ocsps
 
-            crls = validation_context_kwargs['crls'] = []
+            crls = list(validation_context_kwargs.pop('crls', ()))
             for crl_ref in self.crls:
                 crl_stream: generic.StreamObject = crl_ref.get_object()
                 crl = asn1_crl.CertificateList.load(crl_stream.data)
                 crls.append(crl)
+            validation_context_kwargs['crls'] = crls
 
         return ValidationContext(
             other_certs=certs, **validation_context_kwargs
@@ -2081,7 +2116,7 @@ class DocumentSecurityStore:
         try:
             dss_ref = handler.root.raw_get(pdf_name('/DSS'))
         except KeyError as e:
-            raise ValidationInfoReadingError("No DSS found") from e
+            raise NoDSSFoundError() from e
 
         dss_dict = dss_ref.get_object()
 
@@ -2138,7 +2173,7 @@ class DocumentSecurityStore:
         The result is applied to the output stream as an incremental update.
 
         You can either specify the CMS objects to include directly, or
-        pass them in as output from `certvalidator`.
+        pass them in as output from `pyhanko_certvalidator`.
 
         :param output_stream:
             Output stream to write to.
